/*
 * (c) Copyright 2008-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License.
 *
 * For redistributing this software or a derivative work under a license other
 * than the GPL-compatible Free Software License as defined by the Free
 * Software Foundation or approved by OSI, you must first obtain a commercial
 * license to this software product from Volker Bergmann.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.jdbacl.dialect;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Timestamp;

import org.databene.jdbacl.DatabaseDialect;
import org.databene.jdbacl.model.DBSequence;

/**
 * Implements generic database concepts for PostgreSQL.<br/><br/>
 * Created: 26.01.2008 07:11:06
 * @since 0.4.0
 * @author Volker Bergmann
 */
public class PostgreSQLDialect extends DatabaseDialect {
    
	private static final String DATE_PATTERN = "'date '''yyyy-MM-dd''";
	private static final String TIME_PATTERN = "'time '''HH:mm:ss''";

    public PostgreSQLDialect() {
	    super("postgres", false, true, DATE_PATTERN, TIME_PATTERN);
    }

    public DBSequence[] querySequences(Connection connection) throws SQLException {
		throw new UnsupportedOperationException(); // TODO v0.6.13 implement PostgreSQLDialect.querySequences()
	}

    @Override
    public boolean isDefaultCatalog(String catalog, String user) {
        return "".equals(catalog) || user.equalsIgnoreCase(catalog);
    }
    
    @Override
    public boolean isDefaultSchema(String schema, String user) {
        return "public".equalsIgnoreCase(schema);
    }
    
	@Override
    public String renderFetchSequenceValue(String sequenceName) {
        return "select nextval('" + sequenceName + "')";
    }

	@Override
	public String formatTimestamp(Timestamp timestamp) {
	    return "timestamp " + super.formatTimestamp(timestamp);
	}

	@Override
	public boolean isDeterministicPKName(String pkName) {
		return true; // PostgreSQL generates deterministic names 
	}

	@Override
	public boolean isDeterministicUKName(String ukName) {
		return true; // PostgreSQL generates deterministic names
	}

	@Override
	public boolean isDeterministicFKName(String fkName) {
		return true; // PostgreSQL generates deterministic names
	}

	@Override
	public boolean isDeterministicIndexName(String indexName) {
		return true; // PostgreSQL generates deterministic names
	}

	@Override
	public boolean supportsRegex() {
		return true;
	}
	
	@Override
	public String regexQuery(String expression, boolean not, String regex) {
		return (not ? "NOT " : "") + expression + " ~ '" + regex + "'";
	}

}
